@@ -1,10 +1,8 @@
-from http import HTTPStatus as status
-
 from collections import deque
+from functools import wraps
 from inspect import iscoroutinefunction
 
-from uvicorn import run
-from typing import Callable, Generic, TypeVar
+from typing import Callable
 
 from .constants import (
     CONNECT,
@@ -12,8 +10,6 @@ from .constants import (
     DELETE,
     GET,
     HEAD,
-    INITIALIZATION_MESSAGE,
-    MESSAGE_NOT_FOUND,
     METHODS,
     METHOD_CONNECT,
     METHOD_DELETE,
@@ -27,7 +23,6 @@ from .constants import (
     PATCH,
     POST,
     PUT,
-    STATUS_NOT_FOUND,
     TRACE,
     URL_ERROR_MESSAGE
 )
@@ -85,9 +80,16 @@ class Route(object):
                 current_node = node.children.get(':')
                 if current_node:
                     r.params = current_node.parameterized, route
+                    """Store the label that immediately follows the ':' represented by paremeterized
+                    and its value represented as route into request.params"""
+
                     if(node.children.get('*')):
                         route_at_deviation = '/'.join([route, *routes])
                         deviation_point = node.children.get('*')
+                    """If there was also a wildcard seeing as placeholder ':' takes precedence, then
+                    mark the point it deviated so it is possible to backtrack and use that point for
+                    matching if this path fumbles later"""
+
                     node = current_node
                     continue
 
@@ -99,6 +101,8 @@ class Route(object):
                 if deviation_point:
                     r.params = '*', route_at_deviation
                     return deviation_point.route, deviation_point.handler
+                """This is one place where the deviation point is used - the remainder of the url supplied is
+                passed into request.params as the value for '*' for optional lookup purposes"""
 
                 return matched, self.not_found
             node = current_node
@@ -146,7 +150,8 @@ class Routes(object):
             route_node.handler = handler
             return
 
-        # Otherwise strip and split the routes into stops or stoppable stumps i.e. /customers /:id /orders
+        # Otherwise strip and split the routes into stops or stoppable
+        # stumps i.e. /customers/:id/orders -> [customers, :id, orders]
         routes = route.strip(SEPARATOR).split(SEPARATOR)
 
         # get the length of the routes so we can use for validation checks in a loop later
@@ -201,7 +206,12 @@ class Routes(object):
         """
         Traverse internal route tree and use appropriate method
         """
-        body = await receive()
+        body = b''
+        more = True
+        while more:
+            msg = await receive()
+            body += msg.get('body', b'')
+            more = msg.get('more_body', False)
 
         r = HttpRequest(scope, body, receive, metadata, application)
         w = ResponseWriter()
@@ -276,15 +286,24 @@ class Routes(object):
                 self.cache[method][route] = None
 
     async def xhooks(self, hookstore, matched, r: HttpRequest, w: ResponseWriter, c: Context):
-        # traverse the before tree - changed to tree to match routes tree
-        # for every * add it to list of hooks to call
-        # if match also add functions there to list of hooks to call
         hooks = set(hookstore.get(matched, []))
+        """Here we are getting any and or all hooks that match the url verbatim as provided
+        i.e. /url/:id or /url/:id/nested
+        """
+
         parts = matched.strip(SEPARATOR).split(SEPARATOR)
         for position, part in enumerate(parts):
             joinedparts = "/".join(parts[:position])
             _ = '' if position == 0 else SEPARATOR
             hooks.update(hookstore.get(f'/{joinedparts}{_}*', []))
+        """Next we clean the matched url of any '/' surpluses before splitting it into a list
+        for enumeration.
+        Enumeration helps us gradually use the current enumeration position/offset/index to
+        gradually append '*' until we get a match.
+        We stop at a level before matched/masked url as we already searched for verbatim match
+        hence no index + 1 used
+        """
+
         for hook in hooks:
             if w._abort: raise AbortException
             if iscoroutinefunction(hook): await hook(r, w, c)
@@ -315,7 +334,7 @@ class Router(object):
                     await send({'type': 'lifespan.shutdown.complete'})
 
         metadata = preprocessor(scope)
-        engine = self.subdomains.get(metadata[0]) or self.subdomains.get('*')
+        engine = self.subdomains.get(metadata[0]) or self.subdomains.get(DEFAULT)
 
         response = await engine.handle(scope, receive, send, metadata, self)
         await send({'type': 'http.response.start', 'headers': response.headers, 'status': response.status})
@@ -373,6 +392,9 @@ class Router(object):
     def TRACE(self, route: str, handler: Callable, subdomain=DEFAULT):
         self.abettor(METHOD_TRACE, route, handler, subdomain)
 
+    def ON(self, *args):
+        return self.ONCE(*args)
+
     def ONCE(self, *args):
         arguments = len(args)
         error_message = 'ONCE requires a callable argument as default'
@@ -381,6 +403,7 @@ class Router(object):
         except: raise TypeError('ONCE function received more than 2 arguments')
 
         def closure(func):
+            @wraps(func)
             async def hidden(r):
                 if iscoroutinefunction(func): return await func(self)
                 else: return func(self)
@@ -394,13 +417,13 @@ class Router(object):
         else:
             first, second = args
 
-            try: assert first in ['startup', 'shutdown']
-            except AssertionError: raise ValueError(help_message)
+            try: assert first.lower() in ['startup', 'shutdown']
+            except (AssertionError, TypeError, AttributeError): raise ValueError(help_message)
 
             try: assert isinstance(second, Callable)
             except AssertionError: raise TypeError(error_message)
 
-            if first == 'startup': self.initializers.append(closure(second))
+            if first.lower() == 'startup': self.initializers.append(closure(second))
             else: self.deinitializers.append(closure(second))
 
     async def _register(self):
@@ -458,9 +481,7 @@ class Router(object):
                     handler = cache[route]
                     self.subdomain(subdomain)
                     self.abettor(method, route, handler, subdomain=subdomain, router=router if isolated else self)
-
-                    afters = engine.afters.get(route, [])
-                    self.subdomains[subdomain].afters[route] = [*afters, *self.subdomains[subdomain].afters.get(route, [])]
-
-                    befores = engine.befores.get(route, [])
-                    self.subdomains[subdomain].befores[route] = [*befores, *self.subdomains[subdomain].befores.get(route, [])]
+            for after in engine.afters:
+                self.subdomains[subdomain].afters[after] = [*engine.afters[after], *self.subdomains[subdomain].afters.get(after, [])]
+            for before in engine.befores:
+                self.subdomains[subdomain].befores[before] = [*engine.befores[before], *self.subdomains[subdomain].befores.get(before, [])]
